﻿// Copyright © 2018 COMFILE Technology
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using AdvancedHMIDrivers;
using MfgControl.AdvancedHMI.Drivers.Modbus.RTU;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO.Ports;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace ComfilePiDrivers
{
    public class LinuxModbusRTUCom : ModbusRTUCom
    {
        public LinuxModbusRTUCom()
            :base()
        {
            
        }

        public LinuxModbusRTUCom(IContainer container)
            : base(container)
        {

        }

        string _portNameWindows = "COM1";

        [Browsable(true)]
        public string PortNameWindows
        {
            get { return _portNameWindows; }
            set
            {
                _portNameWindows = value;
                if (!DesignMode)
                {
                    if (Environment.OSVersion.Platform == PlatformID.Unix)
                    {
                        PortName = _portNameWindows;
                    }
                }
            }
        }

        void DisposePort(SerialPort port)
        {
            if (port != null)
            {
                if (port.IsOpen)
                {
                    port.Close();
                }
                port.Dispose();
            }
        }

        ModbusRTUDataLinkLayer DataLinkLayer
        {
            get
            {
                var t = typeof(ModbusRTU);
                var d = t.GetFields(BindingFlags.Static | BindingFlags.NonPublic)[0];
                var v = (ConcurrentDictionary<int, ModbusRTUDataLinkLayer>)d.GetValue(this);
                return v[MyDLLInstance];
            }
        }

        PropertyInfo SerialPortProperty
        {
            get
            {
                return DataLinkLayer.GetType().GetProperty("SerialPort", BindingFlags.NonPublic | BindingFlags.Instance);
            }
        }

        EnhancedSerialPort SerialPort
        {
            get
            {
                var port = SerialPortProperty.GetValue(DataLinkLayer, null);
                if (!(port is EnhancedSerialPort))
                {
                    DisposePort((SerialPort)port);
                    return null;
                }

                return (EnhancedSerialPort)port;
            }
            set
            {
                var port = SerialPort;
                DisposePort(port);

                SerialPortProperty.SetValue(DataLinkLayer, value, null);
            }
        }

        void DetachPort()
        {
            DisposePort(SerialPort);
            SerialPort = null;

            ConnectionEstablished -= LinuxModbusRTUCom_ConnectionEstablished;
            ConnectionClosed -= LinuxModbusRTUCom_ConnectionClosed;
        }

        protected override void CreateDLLInstance()
        {
            base.CreateDLLInstance();

            if (!DesignMode)
            {
                DetachPort();

                SerialPort = new EnhancedSerialPort(PortName, BaudRate, Parity, DataBits, StopBits);

                ConnectionEstablished += LinuxModbusRTUCom_ConnectionEstablished;
                ConnectionClosed += LinuxModbusRTUCom_ConnectionClosed;
            }
        }

        void LinuxModbusRTUCom_ConnectionClosed(object sender, EventArgs e)
        {
            LinuxModbusRTUCom com = (LinuxModbusRTUCom)sender;
            com.SerialPort.Stop();
        }

        void LinuxModbusRTUCom_ConnectionEstablished(object sender, EventArgs e)
        {
            LinuxModbusRTUCom com = (LinuxModbusRTUCom)sender;
            com.SerialPort.Start();
        }

        protected override void RemoveDLLConnection(int instance)
        {
            if (!DesignMode)
            {
                DetachPort();
            }

            base.RemoveDLLConnection(instance);
        }
    }
}
